﻿using System.Configuration;
using EPiServer.Core;
using System;
using System.Collections.Concurrent;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Runtime.Serialization;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;
using VppMigrationTool.Util;
using EPiServer.Framework.Configuration;
using System.Globalization;

namespace VppMigrationTool
{
    public partial class Converter : Form
    {
        RuntimeProxyWrapper _runtimeProxyWrapper = null;
        Dictionary<String, String> extensions = new Dictionary<string,string>();
        public static BackgroundWorker BGThread;

        public Converter()
        {
            InitializeComponent();
            AppDomain.CurrentDomain.AssemblyResolve += CurrentDomain_AssemblyResolve;
            this.Text = this.Text + " " + typeof(Converter).Assembly.GetName().Version.ToString();
        }

        Assembly CurrentDomain_AssemblyResolve(object sender, ResolveEventArgs args)
        {
            if (args.Name.StartsWith("EPiServer"))
            {
                return Assembly.LoadFile(sitePath_txt.Text + "\\bin" + "\\" + new AssemblyName(args.Name).Name + ".dll");
            }
            return null;
        }

        RuntimeProxyWrapper RuntimeProxyWrapper
        {
            get
            {
                if (_runtimeProxyWrapper == null)
                {
                    _runtimeProxyWrapper = new RuntimeProxyWrapper(sitePath_txt.Text);
                    _runtimeProxyWrapper.GetSiteProxy().Initialize(sitePath_txt.Text, "/");
                }
                return _runtimeProxyWrapper;
            }
        }

        private bool ValidateParams()
        {
            if (vpps_lbx.CheckedItems.Count == 0)
            {
                MessageBox.Show("Select at least one Virtual Path Provider to migrate");
                return false;
            }

            if (regFileData_lbx == null && regFileData_lbx.Items.Count == 0)
            {
                MessageBox.Show("Missing file Data type");
                return false;
            }
            return true;

        }

        private void PreConverting()
        {
            status_txt.Clear();
        }

        private void sitemap_btn_Click(object sender, EventArgs e)
        {
            FillSiteData();
        }

        private void FillSiteData()
        {
            try
            {
                connectToSite_btn.Enabled = false;
                connectToSite_btn.Text = "Connecting..";
                start_btn.Enabled = false;
                vpps_lbx.Items.Clear();
                regFileData_lbx.Items.Clear();

                if (_runtimeProxyWrapper != null)
                {
                    _runtimeProxyWrapper.Dispose();
                    _runtimeProxyWrapper = null;
                }
                status_txt.Clear();
                vpps_lbx.Items.AddRange(RuntimeProxyWrapper.GetSiteProxy().GetVppNames().ToArray());
                regFileData_lbx.Items.AddRange(RuntimeProxyWrapper.GetSiteProxy().GetRegiteredFileData().ToArray());

                if (vpps_lbx.Items.Count == 0)
                {
                    MessageBox.Show("Cannot find any virtual path providers to migrate", "Missing virtual path providers", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                }
                else if (regFileData_lbx.Items.Count == 0)
                {
                    MessageBox.Show("Cannot find any content types for files", "Missing content types", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                }
                else
                {
                    start_btn.Enabled = true;
                    for (int i = 0; i < vpps_lbx.Items.Count; i++)
                    {
                        vpps_lbx.SetItemChecked(i, true);
                    }
                }
            }
            catch (Exception e)
            {
                new ExceptionDetails(e).Show();
            }
            finally
            {
                connectToSite_btn.Enabled = true;
                connectToSite_btn.Text = "Connect";
            }
        }

        private void migrate_btn_Click(object sender, EventArgs e)
        {
            if (IsBackGroundThreadRunning())
            {
                MessageBox.Show("Previous job should be completed before start another job.");
                return;
            }
        
            if (String.IsNullOrEmpty(sitePath_txt.Text))
            {
                MessageBox.Show("Site path is missing");
                return;
            }

            if (!ValidateParams())
            {
                return;
            }

            StartConverting();
        }

        private bool IsBackGroundThreadRunning()
        {
            return BGThread != null;
        }

        private void StartConverting()
        {   
            var providers = new List<string>();
            foreach(string providerName in vpps_lbx.CheckedItems)
            {
                providers.Add(providerName);
            }

            ConverterContext converterContext = new ConverterContext()
            {
                ProviderNames = providers.ToArray(),
                KeepIdentity = keepIdentity_cbx.Checked,
                KeepSummary = keepSummary_cbx.Checked,
                ContinueOnError = continueOnError_cbx.Checked,
                RunParallel = runParallel_checkBok.Checked,
                CreateRootFolderPerVpp = rootFolder_cbox.Checked,
                ConvertedItemInfo = new ConvertedItemInfo(providers)
            };

            StatusHandler statusHandler = new StatusHandler();
            StatusWriter sw = new StatusWriter();
            sw.Done += statusHandler_Done;
            sw.StatusTextBox = status_txt;
            sw.SynchronizationContext = SynchronizationContext.Current;
            sw.ShowLog = showLog_cbx.Checked;
            sw.Progress = progressBar1;
            statusHandler.StatusWriter = sw;
            converterContext.StatusEventHandler = statusHandler.SiteProxyInstance_StatusEventHandler;
            RuntimeProxyWrapper.GetSiteProxy().Convert(converterContext);
            progressBar1.MarqueeAnimationSpeed = 20;
        }

        void statusHandler_Done(object sender, StatusEventArgs e)
        {
            var statusWriter = sender as StatusWriter;

            if (_runtimeProxyWrapper != null)
            {
                try
                {
                    _runtimeProxyWrapper.Dispose();
                }
                finally
                {
                    _runtimeProxyWrapper = null;
                }
            }
            
            //Remove succeeded providers
            var fileMap = new ExeConfigurationFileMap { ExeConfigFilename = Path.Combine(sitePath_txt.Text,  "web.config") };
            var config = ConfigurationManager.OpenMappedExeConfiguration(fileMap, ConfigurationUserLevel.None);
            var frameworkSection = config.GetSection("episerver.framework") as EPiServerFrameworkSection;

            foreach (var unifiedVpp in e.ConvertedItemInfo.ProviderStatus)
            {
                if (unifiedVpp.Value.HasValue)
                {
                    if (unifiedVpp.Value.Value == true)
                    {
                        frameworkSection.VirtualPathProviders.Remove(unifiedVpp.Key);
                        statusWriter.Write(new StatusEventArgs(State.Info, String.Format(CultureInfo.InvariantCulture,
                            "VirtualPathProvider named '{0}' was successfully converted and removed from site config", unifiedVpp.Key), e.ConvertedItemInfo));
                    }
                    else
                    {
                        statusWriter.Write(new StatusEventArgs(State.Info, String.Format(CultureInfo.InvariantCulture,
                            "VirtualPathProvider named '{0}' had failures during conversion and is leaved in site config", unifiedVpp.Key), e.ConvertedItemInfo));
                    }
                }
            }

            config.Save(ConfigurationSaveMode.Modified);


            this.Invoke(new Action( () => { start_btn.Enabled = false; }));
        }

        private void Converter_Load(object sender, EventArgs e)
        {
            start_btn.Enabled = false;
        }

        private void browseFolder_Click(object sender, EventArgs e)
        {
            if (folderBrowserDialog1.ShowDialog() == DialogResult.OK)
            {
                if(!File.Exists(Path.Combine(folderBrowserDialog1.SelectedPath, @"bin\episerver.dll")))
                {
                    MessageBox.Show("The selected path is not an EPiServer site, cannot find a bin-folder with an EPiServer.dll", "Invalid folder", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                }
                else
                {
                    sitePath_txt.Text = folderBrowserDialog1.SelectedPath;
                }

            }
        }

        private void vpps_lbx_Click(object sender, EventArgs e)
        {
            for (int i = 0; i < vpps_lbx.Items.Count; i++)
            {
                if (vpps_lbx.GetItemRectangle(i).Contains(vpps_lbx.PointToClient(MousePosition)))
                {
                    switch (vpps_lbx.GetItemCheckState(i))
                    {
                        case CheckState.Checked:
                            vpps_lbx.SetItemCheckState(i, CheckState.Unchecked);
                            break;
                        case CheckState.Indeterminate:
                        case CheckState.Unchecked:
                            vpps_lbx.SetItemCheckState(i, CheckState.Checked);
                            break;
                    }
                }
            }
        }
    }
}
