﻿using EPiServer.Data;
using EPiServer.DataAccess;
using EPiServer.ServiceLocation;
using System;
using System.Collections.Generic;
using System.Data.Common;

namespace VppMigrationTool.Util
{
    internal class SoftLinkUpdater : DataAccessBase
    {
        public SoftLinkUpdater() :
            base(ServiceLocator.Current.GetInstance<IDatabaseHandler>()) { }

        internal void UpdateReferencedContent()
        {
            Dictionary<int, Guid> listsOfGuids = new Dictionary<int, Guid>();
            Database.Execute(() =>
            {
                using (DbCommand cmd = CreateCommand(
                        @"SELECT pkID, LinkUrl 
                         FROM 
                            tblContentSoftlink  
                         WHERE 
                            (fkReferencedContentGUID IS NULL OR fkReferencedContentGUID = CONVERT(UNIQUEIDENTIFIER,'00000000-0000-0000-0000-000000000000')) AND LinkUrl LIKE '~/link/%' "))
                {
                    cmd.CommandType = System.Data.CommandType.Text;
                    using (var reader = cmd.ExecuteReader())
                    {
                        while (reader.Read())
                        {
                            Guid g;
                            string linkUrl = (string)reader["LinkUrl"];
                            if (TryParseGuidFromLinkUrl((string)reader["LinkUrl"], out g))
                            {
                                listsOfGuids.Add((int)reader["pkID"], g);
                            }
                        }
                    }
                }

            });

            foreach (var kv in listsOfGuids)
            {
                UpdateReferencedContent(kv.Key, kv.Value);
            }
        }

        private bool TryParseGuidFromLinkUrl(String linkurl, out Guid g)
        {
            g = Guid.Empty;
            if (!String.IsNullOrEmpty(linkurl) && linkurl.Length > 40 && linkurl.StartsWith("~/link/"))
            {
                var s = linkurl.Substring(7, 32);
                return Guid.TryParseExact(s, "N", out g);
            }
            return false;
        }

        internal void UpdateReferencedContent(int id, Guid contentGuid)
        {
            Database.ExecuteTransaction(() =>
            {
                using (DbCommand cmd = CreateCommand(
                    @"UPDATE 
                            tblContentSoftlink  
                    SET 
	                        tblContentSoftlink.fkReferencedContentGUID = CONVERT(UNIQUEIDENTIFIER,'" + contentGuid.ToString() + "') " +
                    @" WHERE 
                        	(fkReferencedContentGUID IS NULL OR fkReferencedContentGUID = CONVERT(UNIQUEIDENTIFIER,'00000000-0000-0000-0000-000000000000')) AND pkID = " + id))
                {
                    cmd.CommandType = System.Data.CommandType.Text;
                    cmd.ExecuteNonQuery();
                }
            });
        }
    }
}
 