﻿using System;
using System.Collections.Concurrent;
using System.Collections.Generic;
using System.ComponentModel;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading;
using System.Windows.Forms;

namespace VppMigrationTool.Util
{
    [Serializable]
    public class StatusHandler
    {
        public StatusWriter StatusWriter;
        public void SiteProxyInstance_StatusEventHandler(StatusEventArgs e)
        {
            StatusWriter.Write(e);
        }
    }

    public class StatusWriter : MarshalByRefObject
    {
        public SynchronizationContext SynchronizationContext { get; set; }
        public TextBox StatusTextBox { get; set; }
        public ProgressBar Progress { get; set; }
        public bool ShowLog { get; set; }
        public static ConcurrentQueue<StatusEventArgs> _messages = new ConcurrentQueue<StatusEventArgs>();
        public static object _syncObject= new object();
        public void Write(StatusEventArgs statusEventArgs)
        {
            if (Converter.BGThread == null)
            {
                var filename = Path.GetTempPath();
                Converter.BGThread = new BackgroundWorker();
                Converter.BGThread.DoWork += bkgThread_DoWork;
                Converter.BGThread.RunWorkerCompleted += BGThread_RunWorkerCompleted;
                Converter.BGThread.WorkerSupportsCancellation = true;

                var date = DateTime.Now;
                filename = String.Format("MigratingVPP{0}{1}{2}{3}{4}{5}.log", date.Year.ToString(), date.Month.ToString(), date.Day.ToString(), date.Hour.ToString(), date.Minute.ToString(), date.Second.ToString());
                lock (_syncObject)
                {
                    if (!Converter.BGThread.IsBusy)
                        Converter.BGThread.RunWorkerAsync(filename);
                }
            }
            _messages.Enqueue(statusEventArgs);
        }

        void BGThread_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            Converter.BGThread = null;
        }

        void bkgThread_DoWork(object sender, DoWorkEventArgs e)
        {
            while (true)
            {
                Thread.Sleep(100);

                try
                {
                    if (_messages.Any(s => s.IsDone))
                    {
                        LogStatuses(e, Int32.MaxValue);
                        e.Cancel = true;
                        break;
                    }
                    else
                    {
                        LogStatuses(e, 100);
                    }
                }
                catch (Exception ex)
                {
                    throw ex;
                }
            }
        }

        private void LogStatuses(DoWorkEventArgs e, int takeCount)
        {
            var statuses = new List<StatusEventArgs>();

            StatusEventArgs item;
            while (_messages.TryDequeue(out item) && statuses.Count < takeCount)
            {
                statuses.Add(item);
            }
            SynchronizationContext.Post(UpdateUI, statuses);
            SaveToFile(e.Argument.ToString(), statuses);
        }

        private void SaveToFile(string fileName, IEnumerable<StatusEventArgs> statusEventArgs)
        {
            if (!File.Exists(fileName))
            {
                using (File.Open(fileName, FileMode.Append))
                {
                }
            }
            using (StreamWriter w = File.AppendText(fileName))
            {
                foreach (var status in statusEventArgs)
                {
                    w.WriteLine(CreateMessage(status, true));
                }
                w.Flush();
            }
        }

        public event EventHandler<StatusEventArgs> Done;

        private void UpdateUI(object state)
        {
            var statuses = state as IEnumerable<StatusEventArgs>;
            if (ShowLog)
            {
                StringBuilder msg = new StringBuilder();
                foreach (var status in statuses)
                {
                    msg.Append(CreateMessage(status, false));
                }
                StatusTextBox.AppendText(msg.ToString());
                StatusTextBox.Update();
            }

            var doneStatus = statuses.FirstOrDefault(s => s.IsDone);
            if (doneStatus != null)
            {
                Done(this, doneStatus);
                Progress.MarqueeAnimationSpeed = 0;
                MessageBox.Show("Migration has completed.");
            }
        }

        private String CreateMessage(StatusEventArgs statusEventArgs, bool detailed)
        {
            StringBuilder sb = new StringBuilder(512);
            sb.AppendLine(statusEventArgs.Message);
            if (!String.IsNullOrEmpty(statusEventArgs.StackTrace))
            {
                sb.AppendLine(statusEventArgs.StackTrace);
            }
            if (statusEventArgs.IsDone || detailed)
            {
                sb.AppendLine(statusEventArgs.ConvertedItemInfo.ToString());
            }
            return sb.ToString();
        }
    }
}
